use crate::runtime::{Variable, type_key};
use crate::tag;

mod builder; pub use builder::*;

mod boolean; pub use boolean::Boolean;
mod natural; pub use natural::Natural;
mod integer; pub use integer::Integer;
mod block; pub use block::Block;
mod array; pub use array::Array;
mod list; pub use list::List;

pub enum Type {
    Varying,
    Null,
    Boolean(Boolean),
    Natural(Natural),
    Integer(Integer),
    Block(Block),
    Array(Array),
    List(List),
}
impl Type {
    pub fn from(addr:Variable) -> Self
    {
        match unsafe {type_key(addr.class)} {
            tag::VARYING => Type::Varying,
            tag::NULL => Type::Null,
            tag::BOOLEAN => Type::Boolean(Boolean::from(addr)),
            tag::NATURAL => Type::Natural(Natural::from(addr)),
            tag::INTEGER => Type::Integer(Integer::from(addr)),
            tag::ARRAY => Type::Array(Array::from(addr)),
            //tag::LIST => Type::List(List::from(addr)),
            _ => Type::Null,
        }
    }
}
