#[repr(C)]
#[derive(Clone, Copy)]
pub struct Variable {
    pub class:usize,
    pub address:usize,
}

extern "C" {
    pub fn acquire(type_id:usize) -> Variable;
    pub fn release(addr:Variable);

    pub fn type_outer(type_id:usize, key:usize) -> usize;
    pub fn type_inner(type_id:usize) -> usize;
    pub fn type_key(type_id:usize) -> usize;

    pub fn type_size(type_id:usize) -> usize;

    pub fn bool_set(addr:Variable, data:bool);
    pub fn bool_get(addr:Variable) -> bool;
    
    pub fn natural_set(addr:Variable, data:u64);
    pub fn natural_get(addr:Variable) -> u64;

    pub fn integer_set(addr:Variable, data:i64);
    pub fn integer_get(addr:Variable) -> i64;
    
    pub fn block_set(addr:Variable, index:usize, data:u8);
    pub fn block_get(addr:Variable, index:usize) -> u8;

    //pub fn string_set(id:u64, data:Vec<u8>);
    //pub fn string_get(id:u64) -> Vec<u8>;
    //pub fn string_at(id:u64, index:u64) -> u32;
    
    pub fn array_length(addr:Variable) -> usize;
    pub fn array_get(addr:Variable, index:usize) -> Variable;
    pub fn array_set(addr:Variable, index:usize, source:Variable);

    pub fn list_capacity(addr:Variable) -> usize;
    pub fn list_length(addr:Variable) -> usize;
    //pub fn list_first(addr:Variable) -> Variable;
    //pub fn list_last(addr:Variable) -> Variable;
    pub fn list_at(addr:Variable, index:usize) -> Variable;
    pub fn list_clear(addr:Variable);
    //pub fn list_prepend(addr:Variable, src:Variable);
    //pub fn list_append(addr:Variable, src:Variable);
    pub fn list_insert(addr:Variable, index:usize, src:Variable);
    pub fn list_update(addr:Variable, index:usize, src:Variable);
    //pub fn list_truncate(addr:Variable, maximum:usize);
    //pub fn list_shift(addr:Variable, maximum:usize);
    pub fn list_remove(addr:Variable, index:usize, maximum:usize);
    pub fn list_reserve(addr:Variable, capacity:usize);
    //pub fn list_resize(addr:Variable, length:usize);

    //pub fn list_cut(addr:Variable, index:usize, length:usize) -> Variable;

    //pub fn list_find(addr:Variable, target:Variable, start:usize) -> usize;
    //pub fn list_count(addr:Variable, target:Variable, start:usize) -> usize;
}


